import { 
  select, 
  csv,
  event,
  brushX,
  max,
  range,
  format,
  scaleLinear, 
  scaleTime,
  scaleOrdinal,
  schemeCategory10,
  timeParse, 
  timeFormat, 
  extent,
  axisLeft,
  axisBottom,
  line,
  nest
} from 'd3';

export const brushChart = (selection,props) => {
  const {
    width,
    height,
    xValue,
    yValue,
    xAxisLabel,
    yAxisLabel,
    colorValue,
    setBrushCoords,
    xScaleDomain,
    trendline
  } = props;
  
  const margin = { top: 20, bottom: 30, right: 20, left:50};
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;
  const decimalFormat = format("0.2f");
  
  let { data } = props;
  const yScale = scaleLinear()
    .domain(extent(data, d => d.value))
    .range([innerHeight, 0]);

  const xScale = scaleTime()
    .domain(
      xScaleDomain
        ? xScaleDomain
        : extent(data, d => d.Date)
    )
    .range([0, innerWidth]);
  
  const colorScale = scaleOrdinal()
    .domain(['OW2DWL', 'OW2DSWL'])
    .range(['orange', 'blue']);
  
  let g = selection.selectAll('.container').data([null]);
  g = g.enter().append('g').attr('class', 'container')
    .merge(g)
      .attr('transform', `translate(${margin.left},${margin.top})`);
     
  const yAxis = axisLeft(yScale);
  const xAxis = axisBottom(xScale);  
   
  const lineGenerator = line()
    .x(d => xScale(d.Date))
    .y(d => yScale(d.value));
  
  const nested = nest()
    .key(colorValue)
    .entries(data);
 
  colorScale.domain(nested.map(d => d.key));
  
  const yAxisG = g.selectAll('.y-axis').data([null]);
  yAxisG.enter().append('g').attr('class', 'y-axis')
    .merge(yAxisG)
      .call(yAxis);
  
  const xAxisG = g.selectAll('.x-axis').data([null]);
  xAxisG.enter().append('g').attr('class', 'x-axis')
    .merge(xAxisG)
      .attr('transform', `translate(0,${innerHeight})`)
      .call(xAxis);
  
  // xAxisG.append('text')
  //   .attr('class', 'x-axis-label')
  //   .attr('y', 35)
  //   .attr('x', innerWidth/2)
  //   .attr('fill', 'black')
  //   .text(xAxisLabel)
  
 
  selection.selectAll('defs').data([null])
    .enter().append("defs")
    .append("clipPath")
      .attr("id", "clip")
    .append("rect")
      .attr("width", innerWidth)
      .attr("height", innerHeight);
  
  
  const linePath = g.selectAll('.line-path').data(nested);
  linePath
    .enter().append('path')
      .attr('class', 'line-path')
    .merge(linePath)
      .attr('d', d => lineGenerator(d.values))
      .attr('stroke', d => colorScale(d.key));
  
  if(trendline){
    const trendlineBrush = g.selectAll('.trendline').data(trendline);
    trendlineBrush.enter().append('line').attr('class', 'trendline')
     .merge(trendlineBrush)
      .attr('x1', d=> xScale(d[0]))
      .attr('x2', d=> xScale(d[2]))
      .attr('y1', d=> yScale(d[1]))
      .attr('y2', d=> yScale(d[3]));
  
     const trendlineText = g.selectAll('.text-label').data(trendline);
     trendlineText.enter().append('text').attr('class', 'text-label')
       .attr("x", d => 0)
			 .attr("y", d => innerHeight/2)
		     .merge(trendlineText)
           .attr('transform', `translate(${innerWidth/5},${-150})`)
           .text(d => `Eq: ${decimalFormat(d[1])}x 
													 ${decimalFormat(d[3]) >= 0 
                             ? ' + ' 
                             : '  '}${decimalFormat(d[3])}`)
    
  }
  
  const brushed = () => {
    // three equivalent ways to do it.
    
    // setBrushCoords([
    //   xScale.invert(event.selection[0]),
    //   xScale.invert(event.selection[1])
    // ]);
    
    // setBrushCoords(event.selection.map(x => {
    //   return xScale.invert(x);
    // }));

    if (event.selection) {
      setBrushCoords(event.selection.map(xScale.invert));
    }
  }
 
  
  const brush = brushX()
    .extent([[0,0], [innerWidth,innerHeight]])
    .on("brush end", brushed);
  
  if (setBrushCoords) {
    g.call(brush);
  }
  //   .call(brush.move, xScale.range());
   
  
}