// You can import API functions like this from D3.js.
import { 
  select, 
  csv,
  event,
  range,
  brushX,
  timeParse, 
  timeFormat,
  scaleOrdinal
} from 'd3';

//import { displayChart } from './displayChart';
import { brushChart } from './brushChart';
import { leastSquares } from './Regression';
import { colorLegend } from './colorLegend';

const colorScale = scaleOrdinal()
  .domain(['OW2DWL', 'OW2DSWL'])
  .range(['orange', 'blue']);

const svg1 = select('#main-chart');
const svg2 = select('#brush-chart');
svg1.append('g')
  .attr('transform', `translate(700,20)`)
  .call(colorLegend,({
    colorScale,
    circleRadius: 15,
    spacing: 35,
    textOffset: 30
  })
);

const parseDate = timeParse("%d/%m/%Y");
const formatDate = timeFormat("%b %Y");

let data=[];

let brushCoords = null;
let filteredData = null;
let trendline = null;


const render = () => {
  brushChart(svg1, {
    width: +svg1.attr('width'),
    height: +svg1.attr('height'),
    data,
    colorValue: d => d.wl,
    circleRadius: 2,
    xAxisLabel: 'Date',
    yAxisLabel: 'Water Level',
    xScaleDomain: brushCoords,
    trendline
  });
  
  brushChart(svg2, {
    width: +svg2.attr('width'),
    height: +svg2.attr('height'),
    data,
    colorValue: d => d.wl,
    circleRadius: 2,
    xAxisLabel: 'Date',
    yAxisLabel: 'Water Level',
    setBrushCoords: coords => {
      brushCoords = coords;
      filteredData = data
        .filter(d => d.Date >= brushCoords[0] && d.Date <= brushCoords[1]);
      trendline = leastSquares(filteredData);
      render();
    }
  })
}

const wls =["OW2DWL", "OW2DSWL"];
csv('https://vizhub.com/careyshan/datasets/monthlyViz.csv')
  .then(loadedData => {
    loadedData.forEach(row => {
      wls.forEach(wl => {
        if(row[wl] !== undefined && row[wl] !== ''){
          data.push({
            Date:parseDate(row.Date),
            wl,
            value: +row[wl]
          });
        }
      });
    })
  });

setTimeout(()=>{
	render();
}, 2000)